@@ -34,9 +34,9 @@ Horizontal alignment for the displayed image.
 --place
 Choose where on the screen to display the image. The image will
 be scaled to fit into the specified rectangle. The syntax for
-specifying rectanges is <:italic:`width`>x<:italic:`height`>@<:italic:`left`>x<:italic:`top`>. All measurements
-are in cells (i.e. cursor positions) with the origin :italic:`(0, 0)` at
-the top-left corner of the screen.
+specifying rectangles is <:italic:`width`>x<:italic:`height`>@<:italic:`left`>x<:italic:`top`>.
+All measurements are in cells (i.e. cursor positions) with the
+origin :italic:`(0, 0)` at the top-left corner of the screen.
 
 
 --scale-up
@@ -81,6 +81,19 @@ type=bool-set
 Print out the window size as :italic:`widthxheight` (in pixels) and quit. This is a
 convenience method to query the window size if using kitty icat from a
 scripting language that cannot make termios calls.
+
+
+--stdin
+type=choices
+choices=detect,yes,no
+default=detect
+Read image data from stdin. The default is to do it automatically, when STDIN is not a terminal,
+but you can turn it off or on explicitly, if needed.
+
+
+--silent
+type=bool-set
+Do not print out anything to stdout during operation.
 '''
 
 
@@ -174,7 +187,7 @@ def show(outfile, width, height, fmt, transmit_mode='t', align='center', place=N
         write_chunked(cmd, data)
 
 
-def process(path, args):
+def process(path, args, is_tempfile):
     m = identify(path)
     ss = screen_size()
     available_width = args.place.width * (ss.width / ss.cols) if args.place else ss.width
@@ -183,7 +196,7 @@ def process(path, args):
     needs_scaling = needs_scaling or args.scale_up
     if m.fmt == 'png' and not needs_scaling:
         outfile = path
-        transmit_mode = 'f'
+        transmit_mode = 't' if is_tempfile else 'f'
         fmt = 100
         width, height = m.width, m.height
     else:
@@ -249,9 +262,51 @@ def parse_place(raw):
 help_text = (
         'A cat like utility to display images in the terminal.'
         ' You can specify multiple image files and/or directories.'
-        ' Directories are scanned recursively for image files.'
+        ' Directories are scanned recursively for image files. If STDIN'
+        ' is not a terminal, image data will be read from it as well.'
+        ' You can also specify HTTP(S) or FTP URLs which will be'
+        ' automatically downloaded and displayed.'
 )
-usage = 'image-file ...'
+usage = 'image-file-or-url-or-directory ...'
+
+
+def process_single_item(item, args, url_pat=None, maybe_dir=True):
+    is_tempfile = False
+    try:
+        if isinstance(item, bytes):
+            tf = NamedTemporaryFile(prefix='stdin-image-data-', delete=False)
+            tf.write(item), tf.close()
+            item = tf.name
+            is_tempfile = True
+        if url_pat is not None and url_pat.match(item) is not None:
+            from urllib.request import urlretrieve
+            with NamedTemporaryFile(prefix='url-image-data-', delete=False) as tf:
+                try:
+                    urlretrieve(item, filename=tf.name)
+                except Exception as e:
+                    raise SystemExit('Failed to download image at URL: {} with error: {}'.format(item, e))
+                item = tf.name
+            is_tempfile = True
+            process(item, args, is_tempfile)
+        elif item.lower().startswith('file://'):
+            from urllib.parse import urlparse
+            from urllib.request import url2pathname
+            item = urlparse(item)
+            if os.sep == '\\':
+                item = item.netloc + item.path
+            else:
+                item = item.path
+            item = url2pathname(item)
+            process(item, args, is_tempfile)
+        else:
+            if maybe_dir and os.path.isdir(item):
+                for (x, mt) in scan(item):
+                    process_single_item(x, args, url_pat=None, maybe_dir=False)
+            else:
+                process(item, args, is_tempfile)
+    finally:
+        if is_tempfile:
+            os.remove(item)
 
 
 def main(args=sys.argv):
@@ -267,6 +322,14 @@ def main(args=sys.argv):
 
     if not sys.stdout.isatty():
         sys.stdout = open(os.ctermid(), 'w')
+    stdin_data = None
+    if args.stdin == 'yes' or (not sys.stdin.isatty() and args.stdin == 'detect'):
+        stdin_data = sys.stdin.buffer.read()
+        if stdin_data:
+            items.insert(0, stdin_data)
+        sys.stdin.close()
+        sys.stdin = open(os.ctermid(), 'r')
+
     screen_size = screen_size_function()
     signal.signal(signal.SIGWINCH, lambda signum, frame: setattr(screen_size, 'changed', True))
     if screen_size().width == 0:
@@ -286,7 +349,7 @@ def main(args=sys.argv):
         print('file' if detect_support.has_files else 'stream', end='', file=sys.stderr)
         return
     if args.transfer_mode == 'detect':
-        if not detect_support(wait_for=args.detection_timeout):
+        if not detect_support(wait_for=args.detection_timeout, silent=args.silent):
             raise SystemExit('This terminal emulator does not support the graphics protocol, use a terminal emulator such as kitty that does support it')
     else:
         detect_support.has_files = args.transfer_mode == 'file'
@@ -298,16 +361,13 @@ def main(args=sys.argv):
     if not items:
         raise SystemExit('You must specify at least one file to cat')
     if args.place:
-        if len(items) > 1 or os.path.isdir(items[0]):
-            raise SystemExit('The --place option can only be used with a single image')
+        if len(items) > 1 or (isinstance(items[0], str) and os.path.isdir(items[0])):
+            raise SystemExit(f'The --place option can only be used with a single image, not {items}')
         sys.stdout.buffer.write(b'\0337')  # save cursor
+    url_pat = re.compile(r'(?:https?|ftp)://', flags=re.I)
     for item in items:
         try:
-            if os.path.isdir(item):
-                for x in scan(item):
-                    process(item, args)
-            else:
-                process(item, args)
+            process_single_item(item, args, url_pat)
         except NoImageMagick as e:
             raise SystemExit(str(e))
         except ConvertFailed as e:
